#!/usr/bin/env python

#
# Copyright 2023 Chris Josephes
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#


'''
Primary media validator tests.
'''

# pylint: disable=R0801


import re
from media.validation.core.faults import FaultLevel, Fault
from media.validation.core.status import StatusCode, TestResult
from media.validation.core.tests import AbstractValidator
from media.validation.tests.media.contents import ValidateContentTests


class MediaValidator(AbstractValidator):
    '''
    Main validator object for media.
    '''
    def __init__(self):
        super().__init__()
        self.load_objects()

    def load_objects(self):
        '''
        Load all related testing objects.
        '''
        # self.objects.append(MediaTitleTests())
        self.objects.append(MediaTitleValidator())
        self.objects.append(ValidateContentTests())

    def run_standard_tests(self, in_dish, in_level=5):
        '''
        Run standard media tests.
        '''
        self.local_tests = [self.test_title_present]
        for l_test in self.local_tests:
            l_test(in_dish)
        for tst in self.objects:
            tst.run_standard_tests(in_dish, in_level)

    def test_title_present(self, in_dish):
        '''
        Test the presence of the title element.
        '''
        test_r = TestResult('media.title', 5)
        title_o = in_dish.media.title
        if title_o:
            test_r.autopass()
        else:
            fault = Fault(FaultLevel.CRITICAL, 'No title element')
            test_r.result(0, StatusCode.FAIL, fault)
        in_dish.status.add_result(test_r)


class MediaTitleValidator(AbstractValidator):
    '''
    Validation tests for the media title.
    '''

    def run_standard_tests(self, in_dish, in_level):
        '''
        Run all title related tests.
        '''
        self.test_title_main(in_dish)
        self.test_title_main_value(in_dish)
        self.test_title_edition(in_dish)
        self.test_title_edition_value(in_dish)
        if in_level <= 4:
            self.test_title_main_whitespace(in_dish)
            self.test_title_edition_whitespace(in_dish)

    def test_title_main(self, in_dish):
        '''
        Test the presence of the main title.
        '''
        test_r = TestResult('media.title.main', 5)
        tobj = in_dish.media.title
        if tobj:
            if tobj.title:
                test_r.autopass()
            else:
                fault = Fault(FaultLevel.WARNING, 'No main title')
                test_r.result(0, StatusCode.FAIL, fault)
        else:
            test_r.skip()
        in_dish.status.add_result(test_r)

    def test_title_main_value(self, in_dish):
        '''
        Test the value of the main title.
        '''
        test_r = TestResult('media.title.main.value', 5)
        tobj = in_dish.media.title
        if tobj:
            if tobj.title:
                t_str = tobj.title.strip()
                if t_str == '':
                    fault = Fault(FaultLevel.WARNING, 'Main title empty')
                    test_r.result(0, StatusCode.FAIL, fault)
                else:
                    test_r.autopass()
            else:
                test_r.skip()
        else:
            test_r.skip()
        in_dish.status.add_result(test_r)

    def test_title_main_whitespace(self, in_dish):
        '''
        Test for whitespace in the main title.
        '''
        test_r = TestResult('media.title.main.whitespace', 5)
        tobj = in_dish.media.title.title
        if tobj:
            faults = self._whitespace_test(str(tobj))
            if len(faults) > 0:
                test_r.result(0, StatusCode.FAIL)
                test_r.add_faults(faults)
            else:
                test_r.autopass()
        else:
            test_r.skip()
        in_dish.status.add_result(test_r)

    def test_title_edition(self, in_dish):
        '''
        Test the presence of an edition element.
        '''
        test_r = TestResult('media.title.edition', 5)
        tobj = in_dish.media.title
        if tobj:
            if tobj.edition:
                test_r.autopass()
            else:
                test_r.skip()
        else:
            test_r.skip()
        in_dish.status.add_result(test_r)

    def test_title_edition_value(self, in_dish):
        '''
        Validate the edition value of a piece of media.
        '''
        test_r = TestResult('media.title.edition.value', 5)
        tobj = in_dish.media.title
        if tobj:
            if tobj.edition:
                e_str = tobj.edition.strip()
                if e_str == '':
                    fault = Fault(FaultLevel.WARNING, 'Edition title empty')
                    test_r.result(0, StatusCode.FAIL, fault)
                else:
                    test_r.autopass()
            else:
                test_r.skip()
        else:
            test_r.skip()
        in_dish.status.add_result(test_r)

    def test_title_edition_whitespace(self, in_dish):
        '''
        Test for whitespace in the edition title.
        '''
        test_r = TestResult('media.title.edition.whitespace', 5)
        tobj = in_dish.media.title.edition
        if tobj:
            faults = self._whitespace_test(str(tobj))
            if len(faults) > 0:
                test_r.result(0, StatusCode.FAIL)
                test_r.add_faults(faults)
            else:
                test_r.autopass()
        else:
            test_r.skip()
        in_dish.status.add_result(test_r)

    def _whitespace_test(self, in_str):
        faults = []
        l_pad = re.search(r"^\s+", in_str)
        r_pad = re.search(r"\s+$", in_str)
        if l_pad:
            fault = Fault(FaultLevel.WARNING, 'Leading whitespace')
            faults.append(fault)
        if r_pad:
            fault = Fault(FaultLevel.WARNING, 'Trailing whitespace')
            faults.append(fault)
        return faults



#!/usr/bin/env python

#
# Copyright 2025 Chris Josephes
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#

'''
Code for HTML table output.
'''

# pylint:disable=R0903
# pylint:disable=R0801

from media.fmt.formatter.html.common import sanitize_for_xml
from media.fmt.structure.table import (TableHeaderColumnSpec,
                                       TableSetupException)


class Table():
    '''
    HTML table class.
    '''
    def __init__(self):
        self.count = 0

    def render(self, in_table):
        '''
        Generate a HTML table.
        '''
        if len(in_table.col_specs) == 0:
            raise TableSetupException('No columns defined')
        output = "<table>\n"
        output += self._render_column_setup(in_table)
        output += self._render_headers(in_table)
        output += self._render_body(in_table)
        output += "</table>\n"
        self.count += 1
        return output

    def _render_column_setup(self, in_table):
        output = ''
        output += " <colgroup>\n"
        for col_iter in in_table.col_specs:
            if isinstance(col_iter, TableHeaderColumnSpec):
                output += "  <col class='headers'/>\n"
            else:
                output += "  <col/>\n"
        output += " </colgroup>\n"
        return output

    def _render_headers(self, in_table):
        '''
        We want to output a the header row, but
        only if the table has defined headers.
        '''
        output = ''
        has_hdr_text = False
        for col_iter in in_table.col_specs:
            if col_iter.hdr_text:
                has_hdr_text = True
                break
        if has_hdr_text:
            output = " <thead>\n  <tr>\n"
            for col_iter in in_table.col_specs:
                output += f"   <th>{col_iter.hdr_text}</th>\n"
            output += "  </tr>\n </thead>\n"
        return output

    def _render_body(self, in_table):
        output = ''
        for chunk in in_table.bchunks:
            output += TableBodyChunk.render(chunk)
        return output


class TableBodyChunk():
    '''
    The TableBodyChunk represents a group of table rows.

    Most of the init code is handled by the parent class.
    '''

    @classmethod
    def render(cls, in_chunk):
        '''
        Generate a group of table rows.
        '''
        output = " <tbody>\n"
        col_count = len(in_chunk.table.col_specs)
        if in_chunk.body_header:
            output += "  <tr>\n"
            output += f"   <th colspan='{col_count}'>"
            output += f"{in_chunk.body_header}</th>\n  </tr>\n"
        for row in in_chunk.rows:
            output += TableBodyRow.render(row)
        output += " </tbody>\n"
        return output


class TableBodyRow():
    '''
    Most of the init code is handled by the parent class.
    '''

    @classmethod
    def render(cls, in_row):
        '''
        Generate a table row.
        '''
        output = "  <tr>\n"
        for cell in in_row.cells:
            output += TableCell.render(cell)
        output += "  </tr>\n"
        return output


class TableCell():
    '''
    CSV Table Cell.

    A cell will contain both the value it needs,
    and the specification information on how to render it.
    '''

    @classmethod
    def render(cls, in_cell):
        '''
        Generate a single table cell.
        '''
        if isinstance(in_cell.spec, TableHeaderColumnSpec):
            output = f"   <th>{sanitize_for_xml(in_cell.value)}</th>\n"
        else:
            output = f"   <td>{sanitize_for_xml(in_cell.value)}</td>\n"
        return output



#!/usr/bin/env python

#
# Copyright 2025 Chris Josephes
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#

'''Common code related to content object sorting and grouping.'''

# pylint: disable=R0801
# pylint: disable=too-few-public-methods


import random
from media.general.sorting.groups import (
        GroupAll, GroupTitleAlphabetical,
        GroupDecade, GroupPrimaryGenre)


class AbstractOrganizer():
    '''
    The AbstractOrganizer class takes a list of content objects and creates
    an output list of one or more batch objects, which contain the same
    content, collated in a desired structure.  IE, for a list of 30
    movies, the class will examine each movie, and divide it into
    batches based on criteria like the first letter of the title,
    the genre, or the decade it was made.

    This is an abstract class not meant to be instantiated.  Subclasses
    can add additional organization criteria.
    '''
    G_NONE = 1
    G_ANY_ALPHA = 2
    G_ANY_DECADE = 3
    G_ANY_GENRE = 4

    def __init__(self, in_list, grouping=None):
        self.batches = {}
        self.grouping = grouping
        self.entries = []
        self.working = []
        self.original_count = len(in_list)
        self._add_index_objects(in_list)
        if self.grouping is not None:
            self.grouping = AbstractOrganizer.G_NONE

    def _add_index_objects(self, in_list):
        for con in in_list:
            self.entries.append(con.s_index)

    @classmethod
    def get_random_sample(cls, in_list, limit):
        '''
        Take a large list, and turn it into a random sample.

        Find the total number of objects in the list.
        If the sample limit is greater than the number
        of objects, than just return the original list.

        Otherwise, generate a 'limit' number of random
        numbers, between 0 and the list total and put them
        in an array.  Then iterate through that array, using
        each random number as an index value against the
        original list.
        '''
        total = len(in_list)-1
        if limit > total:
            return in_list
        slots = []
        out = []
        while len(slots) <= limit-1:
            r_num = random.randint(0, total)
            if r_num not in slots:
                slots.append(r_num)
        for slot_i in slots:
            out.append(in_list[slot_i])
        return out

    def set_grouping(self, in_group):
        '''
        Set the grouping attribute value.
        '''
        self.grouping = in_group

    def create_batches(self, grouping=None, sample_limit=None):
        '''
        Create the Batch objects.
        '''
        if grouping:
            self.grouping = grouping
        if sample_limit:
            self.working = AbstractOrganizer.get_random_sample(self.entries,
                                                               sample_limit)
        else:
            self.working = self.entries
        self._build_batches()
        return list(self.batches.values())

    def _build_batches(self):
        if self.grouping == AbstractOrganizer.G_NONE:
            self.batches = GroupAll.group(self.working)
        elif self.grouping == AbstractOrganizer.G_ANY_ALPHA:
            self.batches = GroupTitleAlphabetical.group(self.working)
        elif self.grouping == AbstractOrganizer.G_ANY_DECADE:
            self.batches = GroupDecade.group(self.working)
        elif self.grouping == AbstractOrganizer.G_ANY_GENRE:
            self.batches = GroupPrimaryGenre.group(self.working)
        else:
            self.batches = GroupAll.group(self.working)


class Organizer(AbstractOrganizer):
    '''
    Direct subclass of AbstractOrganizer, with no
    additions or changes, meant to be used as the
    default organizer choice.  Use this class if
    there is no additional organiztion criteria.
    '''



#!/usr/bin/env python

#
# Copyright 2025 Chris Josephes
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#

'''
Code to handle technical aspects of a vidsual format media
(movies, television)
'''

# pylint: disable=too-few-public-methods

from media.xml.namespaces import Namespaces
from media.data.media.contents.genericv.technical import Technical


class VariantPool():
    '''
    A class to handle variant loading.
    '''
    @classmethod
    def read_variants(cls, in_element):
        '''
        Read a variants element structure and create
        the variant child objects.
        '''
        variants = []
        if in_element is not None:
            for child in in_element:
                tagname = Namespaces.ns_strip(child.tag)
                if tagname == 'original':
                    variants.append(OriginalVariant(child))
                elif tagname == 'variant':
                    variants.append(Variant(child))
        return variants


class AbstractVariant():
    '''
    Abstract Variant class.
    '''
    def __init__(self):
        self.id = ''
        self.name = ''
        self.notes = ''

    def _process_xml_stream(self, in_element):
        if 'id' in in_element.attrib:
            self.id = in_element.attrib['id']
        for child in in_element:
            tagname = Namespaces.ns_strip(child.tag)
            if tagname == 'name':
                self.name = child.text
            elif tagname == 'notes':
                self.notes = child.text


class OriginalVariant(AbstractVariant):
    '''
    Original type Variant class
    '''
    def __init__(self, in_element):
        super().__init__()
        if in_element is not None:
            self._process_xml_stream(in_element)


class Variant(AbstractVariant):
    '''
    Regular type Variant class.
    '''

    def __init__(self, in_element):
        super().__init__()
        self.techincal = None
        if in_element is not None:
            self._process_xml_stream(in_element)

    def _process_xml_stream(self, in_element):
        super()._process_xml_stream(in_element)
        for child in in_element:
            tagname = Namespaces.ns_strip(child.tag)
            if tagname == 'technical':
                self.technical = Technical(child)



#!/usr/bin/env python

#
# Copyright 2024 Chris Josephes
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#

'''
Formatting functions for basic plain text output.
'''

import textwrap


DEF_WIDTH = 76


def hdr_list(header, i_list):
    '''
    Output a list of items with a header
    '''
    out_string = ""
    if len(i_list) == 1:
        out_string = f"{header}: {i_list[0]}"
    else:
        hdr_string = f"{header}s: "
        bfr_string = " " * len(hdr_string)
        str_list = ', '.join([str(i_name) for i_name in i_list])
        out_ar = textwrap.wrap(str_list, width=DEF_WIDTH,
                               initial_indent=hdr_string,
                               subsequent_indent=bfr_string)
        out_string = "\n".join(out_ar)
    return out_string


def hdr_block(header, i_list):
    '''
    Output a larger list of items with a header
    '''
    out_string = ""
    if len(i_list) == 1:
        out_string = f"{header}:\n{i_list[0]}"
    else:
        hdr_string = f"{header}s:"
        str_list = ', '.join([str(i_name) for i_name in i_list])
        out_ar = textwrap.wrap(str_list, width=DEF_WIDTH,
                               initial_indent=' ',
                               subsequent_indent=' ')
        out_string = hdr_string + "\n" + "\n".join(out_ar)
    return out_string


def hdr_list_np(header, i_list):
    '''
    Output a larger list of items with a header,
    but no pluralization of the header based on
    item count.
    '''
    out_string = ""
    hdr_string = f"{header}: "
    bfr_string = " " * len(hdr_string)
    str_list = ', '.join([str(i_name) for i_name in i_list])
    out_ar = textwrap.wrap(str_list, width=DEF_WIDTH,
                           initial_indent=hdr_string,
                           subsequent_indent=bfr_string)
    out_string = "\n".join(out_ar) + "\n"
    return out_string


def hdr_text(header, text_block):
    '''
    Output a block of text
    '''
    out_string = header + "\n"
    out_ar = textwrap.wrap(text_block, width=DEF_WIDTH)
    out_string += "\n".join(out_ar)
    return out_string


def hdr_list_oneper(header, i_list, indent=0, hformat='s'):
    '''
    Output a list with a header, one item per line.

    header: item1
            item2

    allow for an indent option to pad all output
    and a format specifier if multiple runs of
    hdr_list_oneper need to be aligned at the colon.
    '''
    out = ""
    if len(i_list) == 1:
        f_header = f"{' ' * indent}{header:{hformat}}"
        out = f"{f_header}: {i_list[0]}\n"
    else:
        f_header = f"{' ' * indent}{(header + 's'):{hformat}}"
        bfr_string = " " * len(f_header)
        i_count = 1
        for i_name in i_list:
            if i_count == 1:
                out += f"{f_header}: {i_name}\n"
            else:
                out += f"{bfr_string}  {i_name}\n"
            i_count += 1
    return out



#!/usr/bin/env python

#
# Copyright 2025 Chris Josephes
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#

'''
Object classes for characters in a story.
'''

# pylint: disable=too-few-public-methods


from media.xml.namespaces import Namespaces
from media.general.stringtools import build_sort_string


class AbstractCharacter():
    '''
    Abstract root class of all characters.
    '''
    def __init__(self):
        self.sort_value = ''


class CharacterSelf(AbstractCharacter):
    '''
    A character based on... someone??
    '''
    def __init__(self):
        super().__init__()
        self.name = ''
        # How do we get the actor name here?


class NamedCharacter(AbstractCharacter):
    '''
    A character with a name.
    '''
    def __init__(self):
        super().__init__()
        self.name = ''


class UnnamedCharacter(AbstractCharacter):
    '''
    A character without a name.
    '''
    def __init__(self):
        super().__init__()
        self.name = ''


class CharacterName():
    '''
    Class for the structure of a character name.

    Character names are more loose/chaotic versus
    PersonalNames because details can be intentionally
    missing.

    '''
    name_tags = ['pgn', 'pcn', 'gn', 'nick', 'mn', 'mi', 'fn']
    sort_order = ['fn', 'pgn', 'gn', 'mn', 'mi']

    def __init__(self, in_element):
        self.value = ''
        self.formal = ''
        self.sort_value = ''
        self.matrix = {}
        self.prefix = ''
        self.suffix = ''
        self.addendum = ''
        if in_element is not None:
            self._process(in_element)

    def _process(self, in_element):
        order = []
        for child in in_element:
            tagname = Namespaces.ns_strip(child.tag)
            if tagname in CharacterName.name_tags:
                if tagname == 'nick':
                    self.matrix[tagname] = "'" + child.text + "'"
                else:
                    self.matrix[tagname] = child.text
                order.append(tagname)
            elif tagname == 'prefix':
                self.prefix = child.text
            elif tagname == 'suffix':
                self.suffix = child.text
            elif tagname == 'addendum':
                self.addendum = child.text
        self._build_value(order)
        self._build_formal()
        self._build_sort()

    def _build_value(self, in_order):
        raw = ''
        for tag_name in in_order:
            raw += self.matrix[tag_name] + ' '
        raw = raw[:-1]
        self.value = raw

    def _build_formal(self):
        raw = ''
        if self.prefix:
            raw = self.prefix + ' ' + self.value
        else:
            raw = self.value
        if self.suffix:
            raw += ' ' + self.suffix
        if self.addendum:
            raw += ' (' + self.addendum + ')'
        self.formal = raw

    def _build_sort(self):
        raw = ''
        if 'pcn' in self.matrix:
            raw = build_sort_string(self.matrix['pcn'])
        else:
            for n_sort in CharacterName.sort_order:
                if n_sort in self.matrix:
                    raw += self.matrix[n_sort].casefold() + '_'
            raw = raw[:-1]
        self.sort_value = raw

    def __str__(self):
        return f"{self.value}"

    def __rept__(self):
        return f"{self.matrix}"



#!/usr/bin/env python

#
# Copyright 2023 Chris Josephes
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#


# pylint: disable=too-few-public-methods

'''Authorship record information for media data.'''

from datetime import date, datetime
import xml.etree.ElementTree as ET
from media.xml.namespaces import Namespaces


class AuthorshipRecord():
    """
    Record showing the authorship of a piece of vtmedia data.
    """
    def __init__(self, in_element=None):
        self.title = ''
        self.catalog = None
        self.summary = None
        self.changelog = []
        if in_element:
            self._process(in_element)

    def _process(self, in_element):
        for child in in_element:
            tagname = Namespaces.ns_strip(child.tag)
            if tagname == 'title':
                self.title = child.text.strip()
            elif tagname == 'catalog':
                self.catalog = Catalog(child)
            elif tagname == 'summary':
                self.summary = Summary(child)
            elif tagname == 'changelog':
                for chg_element in child:
                    chg_tag = Namespaces.ns_strip(chg_element.tag)
                    if chg_tag == 'change':
                        self.changelog.append(ChangeRecord(chg_element))
                    elif chg_tag == 'creation':
                        self.changelog.append(CreationRecord(chg_element))

    def to_element(self):
        """
        Construct an Element object from data.
        """
        authorship = ET.Element('authorshipRecord')
        authorship.set('xmlns', Namespaces.ns['authorship'])
        if self.title:
            ET.SubElement(authorship, 'title').text = self.title
        if self.catalog:
            authorship.append(self.catalog.to_element())
        if self.summary:
            authorship.append(self.summary.to_element())
        if self.changelog:
            changelog = ET.Element('changelog')
            for chg in self.changelog:
                changelog.append(chg.to_element())
            authorship.append(changelog)
        return authorship

    def to_xml(self):
        """
        Return direct XML output from Element object.
        """
        author = self.to_element()
        ET.indent(author)
        return ET.tostring(author, xml_declaration=True).decode()


class Catalog():
    """
    Catalog information for the metarecord, including
    a unique identifier, author, and license information.
    """
    def __init__(self, in_element=None):
        self.identity = ''
        self.authors = []
        self.license = None
        if in_element:
            self._process(in_element)

    def _process(self, in_element):
        for child in in_element:
            tagname = Namespaces.ns_strip(child.tag)
            if tagname == 'id':
                self.identity = child.text.strip()
            elif tagname == 'authors':
                for auth_element in child:
                    auth_tag = Namespaces.ns_strip(auth_element.tag)
                    if auth_tag == 'author':
                        self.authors.append(Author(auth_element))
            elif tagname == 'license':
                self.license = License(child)

    def to_element(self):
        """
        Construct an Element object for the data.
        """
        catalog = ET.Element('catalog')
        if self.identity:
            ET.SubElement(catalog, 'id').text = self.identity
        if self.authors:
            alist = ET.Element('authors')
            for auth in self.authors:
                alist.append(auth.to_element())
            catalog.append(alist)
        if self.license:
            catalog.append(self.license.to_element())
        return catalog

    def to_xml(self):
        """
        Return direct XML string.
        """
        return ET.tostring(self.to_element())


class Summary():
    """
    Summary information for an authorship record.
    """
    def __init__(self, in_element=None):
        self.lang = ''
        self.abstract = ''
        self.internal_notes = ''
        if in_element:
            self._process(in_element)

    def _process(self, in_element):
        for child in in_element:
            tagname = Namespaces.ns_strip(child.tag)
            if tagname == 'lang':
                self.lang = child.text.strip()
            elif tagname == 'abstract':
                self.abstract = child.text.strip()
            elif tagname == 'internalNotes':
                self.internal_notes = child.text.strip()

    def to_element(self):
        """
        Construct an element object from the data.
        """
        summary = ET.Element('summary')
        if self.lang:
            ET.SubElement(summary, 'lang').text = self.lang
        if self.abstract:
            ET.SubElement(summary, 'abstract').text = self.abstract
        if self.internal_notes:
            ET.SubElement(summary, 'internalNotes').text = self.internal_notes
        return summary

    def to_xml(self):
        """
        Return raw XML string.
        """
        return ET.tostring(self.to_element())


class Author():
    """
    Identity of the author(s) of a media record.
    """
    def __init__(self, in_element=None):
        self.name = ''
        self.email = ''
        self.url = ''
        self.fingerprint = ''
        if in_element:
            self._process(in_element)

    def _process(self, in_element):
        for child in in_element:
            tagname = Namespaces.ns_strip(child.tag)
            if tagname == 'name':
                self.name = child.text.strip()
            elif tagname == 'email':
                self.email = child.text.strip()
            elif tagname == 'url':
                self.url = child.text.strip()
            elif tagname == 'fingerprint':
                self.fingerprint = child.text.strip()

    def to_element(self):
        """
        Construct an Element object from the data.
        """
        author = ET.Element('author')
        if self.name:
            ET.SubElement(author, 'name').text = self.name
        if self.email:
            ET.SubElement(author, 'email').text = self.email
        if self.url:
            ET.SubElement(author, 'url').text = self.url
        if self.fingerprint:
            ET.SubElement(author, 'fingerprint').text = self.fingerprint
        return author

    def to_xml(self):
        """
        Return raw XML string.
        """
        return ET.tostring(self.to_element())


class License():
    """
    License information for a record.
    """
    def __init__(self, in_element=None):
        self.code = None
        self.notice = ''
        if in_element:
            self._process(in_element)

    def _process(self, in_element):
        for child in in_element:
            tagname = Namespaces.ns_strip(child.tag)
            if tagname == 'code':
                self.code = LicenseCode(child)
            elif tagname == 'notice':
                self.notice = child.text.strip()

    def to_element(self):
        """
        Build the element object with data.
        """
        file_license = ET.Element('license')
        if self.code:
            file_license.append(self.code.to_element())
        if self.notice:
            ET.SubElement(file_license, 'notice').text = self.notice
        return file_license

    def to_xml(self):
        """
        Return the XML as a string.
        """
        return ET.tostring(self.to_element())


class LicenseCode():
    """
    A shortened code representation of a license.
    Could be SPDX, but could be something else down the road.
    """
    def __init__(self, in_element=None):
        self.type = ''
        self.text = ''
        if in_element:
            self.text = in_element.text.strip()
            if 'type' in in_element.attrib:
                self.type = in_element.attrib['type'].strip()

    def to_element(self):
        """
        Built the Element object with data.
        """
        code = ET.Element('code')
        if self.text:
            code.text = self.text
        if self.type:
            code.set('type', self.type)
        return code

    def to_xml(self):
        """
        Return the XML as a string.
        """
        return ET.tostring(self.to_element())


class AbstractChangeRecord():
    """
    Abstract class representing a change record.
    """
    def __init__(self, in_element=None):
        self.tstamp = None
        self.ticket = None
        self.authors = []
        self.notes = ''
        if in_element:
            self._process(in_element)

    def _process(self, in_element):
        for child in in_element:
            tagname = Namespaces.ns_strip(child.tag)
            if tagname == 'date':
                self.tstamp = date.fromisoformat(child.text.strip())
            elif tagname == 'timestamp':
                self.tstamp = datetime.fromisoformat(child.text.strip())
            elif tagname == 'ticket':
                self.ticket = ChangeTicket(child)
            elif tagname == 'authors':
                for auth_element in child:
                    auth_tag = Namespaces.ns_strip(auth_element.tag)
                    if auth_tag == 'author':
                        self.authors.append(Author(auth_element))
            elif tagname == 'notes':
                self.notes = child.text.strip()

    def build_element(self, record):
        """
        Build the change record object.
        """
        if self.tstamp:
            date_element = None
            if isinstance(self.tstamp, date):
                date_element = ET.SubElement(record, 'date')
            elif isinstance(self.tstamp, datetime):
                date_element = ET.SubElement(record, 'timestamp')
            date_element.text = self.tstamp.isoformat()
        if self.ticket:
            record.append(self.ticket.to_element())
        if self.authors:
            alist = ET.Element('authors')
            for auth in self.authors:
                alist.append(auth.to_element())
            record.append(alist)
        if self.notes:
            ET.SubElement(record, 'notes').text = self.notes
        return record


class ChangeRecord(AbstractChangeRecord):
    """
    Change record reflecting a single change.
    """
    def to_element(self):
        """
        Generate Element object with data.
        """
        record = ET.Element('change')
        super().build_element(record)
        return record

    def to_xml(self):
        """
        Return XML stirng.
        """
        return ET.tostring(self.to_element())


class CreationRecord(AbstractChangeRecord):
    """
    Change record specifically referencing the
    creation of the data record.
    """
    def to_element(self):
        """
        Generate Element object with data.
        """
        record = ET.Element('creation')
        super().build_element(record)
        return record

    def to_xml(self):
        """
        Return XML string.
        """
        return ET.tostring(self.to_element())


class ChangeTicket():
    """
    A change ticket with optional hyperlink to
    provide further details on a change.
    """
    def __init__(self, in_element=None):
        self.ticket_id = ''
        self.href = ''
        if in_element:
            self.ticket_id = in_element.text.strip()
            if 'href' in in_element.attrib:
                self.href = in_element.attrib['href'].strip()

    def to_element(self):
        """
        Generate Element object populated with data.
        """
        ticket = ET.Element('ticket')
        if self.ticket_id:
            ticket.text = self.ticket_id
        if self.href:
            ticket.set('href', self.href)
        return ticket

    def to_xml(self):
        """
        Return XML string of data.
        """
        return ET.tostring(self.to_element())



#!/usr/bin/env python

#
# Copyright 2025 Chris Josephes
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#

'''Main module file for the Movie() and Title() objects'''

# pylint: disable=too-few-public-methods
# pylint: disable=too-many-instance-attributes
# pylint: disable=R0801

from datetime import timedelta
from media.xml.namespaces import Namespaces
from media.data.media.contents import AbstractContent, ContentException
from media.data.media.contents.generic.story import Story
from media.data.media.contents.genericv.crew import Crew
from media.data.media.contents.genericv.technical import Technical
from media.data.media.contents.genericv.variants import VariantPool, Variant
from media.data.media.contents.movie.classification import Classification
from media.general.sorting.index import ContentIndex


class Movie(AbstractContent):
    '''Movie object'''
    def __init__(self, in_element):
        super().__init__()
        self.technical = None
        self.variants = []
        self.crew = None
        self.s_index = None
        self._process(in_element)

    def build_index_object(self):
        """
        Build an index object to make grouping and
        sorting operations easier.
        """
        return MovieIndexEntry(self)

    def _process(self, in_element):
        super()._process(in_element)
        for child in in_element:
            if child.tag == Namespaces.nsf('movie') + 'classification':
                self.classification = Classification(child)
            elif child.tag == Namespaces.nsf('movie') + 'technical':
                self.technical = Technical(child)
            elif child.tag == Namespaces.nsf('movie') + 'story':
                self.story = Story(child)
            elif child.tag == Namespaces.nsf('movie') + 'description':
                self.story = Story(child)
            elif child.tag == Namespaces.nsf('movie') + 'variants':
                self.variants = VariantPool.read_variants(child)
            elif child.tag == Namespaces.nsf('movie') + 'crew':
                self.crew = Crew(child)
        self._post_load_process()

    def _post_load_process(self):
        super()._post_load_process()
        self.s_index = MovieIndexEntry(self)

    def __hash__(self):
        return hash(self.unique_key)

    def __lt__(self, other):
        return self.unique_key < other.unique_key

    def __gt__(self, other):
        return self.unique_key > other.unique_key


class MovieIndexEntry(ContentIndex):
    '''
    A simple indexing object for movie data.
    '''
    def __init__(self, in_movie):
        super().__init__()
        self.movie = in_movie
        self.year = 0
        self.decade = 0
        self.primary_g = None
        self.first_letter = None
        self.runtime = None
        self._extract_fields()

    def _extract_fields(self):

        '''
        Extract all the information we need for organizing.
        '''
        if self.movie.catalog:
            cat = self.movie.catalog
            if cat.copyright:
                self.year = cat.copyright.year
                self.decade = int(self.year / 10)
        if self.movie.classification:
            cls = self.movie.classification
            if cls.genres:
                if cls.genres.primary:
                    self.primary_g = cls.genres.primary
        self._extract_runtime()
        self.sort_title = self.movie.sort_title
        self.first_letter = self.sort_title[0]

    def _extract_runtime(self):
        if self.movie.technical:
            tch = self.movie.technical
            if tch.runtime:
                if tch.runtime.overall:
                    self.runtime = tch.runtime.overall
        if not self.runtime:
            if len(self.movie.variants) > 0:
                self._extract_variant_runtime()
        if not self.runtime:
            self.runtime = timedelta(seconds=0)

    def _extract_variant_runtime(self):
        for var_i in self.movie.variants:
            if issubclass(var_i.__class__, Variant):
                if var_i.technical:
                    tch = var_i.technical
                    if tch.runtime:
                        if tch.runtime.overall:
                            self.runtime = tch.runtime.overall


class MovieException(ContentException):
    '''Exception raised when there is an issue with a movie.'''
    def __init__(self, message):
        super().__init__(message)
        self.message = message

    def __str__(self):
        return self.message



#!/usr/bin/env python

#
# Copyright 2025 Chris Josephes
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#

'''
Standard text format reports for movies.
'''

# pylint: disable=R0801

from datetime import timedelta
from media.fmt.text.basics import hdr_list, hdr_list_np, hdr_text, hdr_block

# Note: The field limit for movie titles should probably be 45.


class OneLiner():
    '''
    Format for a single line summary of a movie.
    '''
    def __init__(self, in_movie):
        self.movie = in_movie
        self.output = ""
        self.build()

    def build(self):
        """
        Construct a one-line summary of a movie.
        """
        y_string = build_copyright_year(self.movie)
        runtime = build_runtime(self.movie)
        category = build_genre_classification(self.movie)
        self.output = f"{self.movie.title!s:50s} " + \
                      f"{y_string:4s} " + \
                      f"{runtime:8s} " + \
                      f"{category:s}\n"

    def __str__(self):
        return self.output

    @classmethod
    def header_fields(cls):
        """
        Return header titles for column output.
        """
        out = f"{'Title':50s} " + \
              f"{'Year':4s} " + \
              f"{'Runtime':8s} " + \
              f"{'Genre':50s}\n" + \
              cls.header_line()
        return out

    @classmethod
    def header_line(cls):
        """
        Return a simple line to go under the header fields.
        """
        return f"{'=' * 50} {'=' * 4} {'=' * 8} {'=' * 50}\n"


class MiniEntry():
    '''Oneliner mini-format'''
    def __init__(self, in_movie, in_indent=2):
        self.movie = in_movie
        self.indent = in_indent
        self._prep_fields()
        self.output = self._build_output()

    @classmethod
    def header_line(cls, in_indent=2):
        '''Generate a simple header'''
        out = f"{' ' * in_indent}{'Title':40s} {'Year':4s} {'Genre':20s}\n" + \
              cls.header_line_line()
        return out

    @classmethod
    def header_line_line(cls, in_indent=2):
        """
        Generate a line to separate headers vs rows.
        """
        return f"{' ' * in_indent}{'-' * 40} {'-' * 4} {'-' * 20}\n"

    def _prep_fields(self):
        self.title_key = self.movie.unique_key
        self.year = ''
        self.genre = ''
        if self.movie.catalog is not None:
            if self.movie.catalog.copyright is not None:
                self.year = self.movie.catalog.copyright.year
        if self.movie.classification is not None:
            self.genre = build_genre_simple(self.movie)

    def _build_output(self):
        return f"{' ' * self.indent}{self.movie.title!s:40s} " + \
               f"{self.year:4d} {self.genre:20s}\n"

    def __str__(self):
        return self.output


class Brief():
    '''Formatting for a brief text record'''
    def __init__(self, in_movie):
        self.movie = in_movie
        self.output = ""
        self.build()

    def build(self):
        """
        Construct output string based on movie data.
        """
        self.output = self.entry_header()
        self.output += self.classification_info()
        self.output += self.primary_crew()
        self.output += self.cast()
        self.output += self.plot()
        self.output += self.keywords() + "\n"

    def entry_header(self):
        '''Basic report header for a movie'''
        y_string = ""
        if self.movie.catalog is not None:
            if self.movie.catalog.copyright is not None:
                y_string = self.movie.catalog.copyright.year
        if y_string:
            out = f"{self.movie.title!s:69s} ({y_string})\n"
        else:
            out = f"{self.movie.title}\n"
        out += "=" * 76
        out += "\n"
        return out

    def classification_info(self):
        '''Basic report on genre information'''
        o_string = ""
        genre_s = build_genre_classification(self.movie)
        subgenre_s = build_subgenre_classifications(self.movie)
        if genre_s:
            o_string += f"{genre_s}\n"
        if subgenre_s:
            o_string += f"{subgenre_s}\n"
        else:
            o_string += "\n"
        return o_string

    def technical_info(self):
        '''Basic technical information'''
        o_string = ""
        rt_s = build_runtime(self.movie)
        if rt_s:
            o_string = f"Runtime: {rt_s}\n"
        return o_string

    def primary_crew(self):
        '''Report on crew basics'''
        out = ""
        if self.movie.crew is not None:
            if self.movie.crew.directors:
                out += hdr_list("Director",
                                self.movie.crew.directors) + "\n"
            if self.movie.crew.cinemap:
                out += hdr_list("Cinemaphotographer",
                                self.movie.crew.cinemap) + "\n"
            if self.movie.crew.music:
                out += self.music() + "\n"
        return out

    def music(self):
        '''Report on movie music.'''
        out = ""
        music = self.movie.crew.music
        if music.composers:
            out += hdr_list("Composer", music.composers) + "\n"
        if music.music:
            out += hdr_list_np("Music", music.music) + "\n"
        return out

    def cast(self):
        '''Report on film cast'''
        out = ""
        names = []
        if self.movie.crew.cast is not None:
            for role_o in self.movie.crew.cast.cast:
                names.append(role_o.actor)
            out = hdr_list_np("Cast", names)
            out += "\n"
        return out

    def plot(self):
        '''Report on plot'''
        out = ""
        if self.movie.story is not None:
            if self.movie.story.plot and str(self.movie.story.plot) != "":
                out += hdr_text("Plot", str(self.movie.story.plot))
                out += "\n\n"
        return out

    def keywords(self):
        '''Report on keywords'''
        out = ""
        if self.movie.story is not None:
            if self.movie.story.keywords:
                if len(self.movie.story.keywords.all()) > 0:
                    out += hdr_block("Keyword",
                                     self.movie.story.keywords.all())
        return out

    def __str__(self):
        return self.output


def build_genre_simple(in_movie):
    '''
    Build a string for all genres.
    '''
    o_string = ''
    classification = in_movie.classification
    if classification.genres:
        if classification.genres.primary:
            o_string = f"{classification.genres.primary}"
        if classification.genres.secondary:
            o_string += '/' + '/'.join(classification.genres.secondary)
    return o_string


def build_genre_classification(in_movie):
    '''
    Build a text classification string.
    '''
    o_string = ""
    classification = in_movie.classification
    if classification.category:
        o_string = "[" + str(classification.category) + "]"
    o_string += " " + build_genre_simple(in_movie)
    if classification.genres.specific:
        class_s = classification.genres.specific.strip()
        if class_s:
            o_string += f" \"{class_s}\""
    return o_string


def build_subgenre_classifications(in_movie):
    '''
    Build the subgenre list.
    '''
    subgenre_s = ", ".join(sorted(in_movie.classification.genres.subgenres))
    if subgenre_s:
        return f"({subgenre_s})\n"
    return ""


def build_copyright_year(in_movie):
    '''
    Construct a printable version of the copyright year.
    '''
    year_s = "0000"
    if in_movie.catalog:
        if in_movie.catalog.copyright:
            year_s = f"{in_movie.catalog.copyright.year:4d}"
    return year_s


def build_runtime(in_movie):
    '''
    Construct a presentable version of the runtime value.
    '''
    runtime = timedelta(seconds=0)
    if in_movie.s_index.runtime:
        runtime = in_movie.s_index.runtime
    return f"{runtime!s}"



#!/usr/bin/env python

#
# Copyright 2025 Chris Josephes
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#

'''
Compare all of the movies against each other.
'''

# pylint: disable=R0801

import os
import argparse
from media.general.compare.movie import MovieComparator
from media.tools.common import load_movies


if __name__ == '__main__':
    parser = argparse.ArgumentParser(description='Simple movie list.')
    parser.add_argument('--mediapath', help='path of media library')
    args = parser.parse_args()
    mediapath = args.mediapath or os.environ['MEDIAPATH']
    if not mediapath:
        parser.print_help()
    all_movies = load_movies(mediapath)
    comparator = MovieComparator()
    comparator.load_data(all_movies)
    comparator.compare()
    comparator.report()



#!/usr/bin/env python

#
# Copyright 2024 Chris Josephes
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#

"""
Simple command to output details on every movie.
"""

# pylint: disable=R0801
# pylint: disable=too-few-public-methods

# brief movie report
import argparse
import media.fmt.text.movie
from media.fileops.filenames import FilenameMatches
from media.fileops.repo import Repo
from media.tools.movies.common import (Controller,
                                       MovieReport)


class ShowController(Controller):
    '''
    Subclass of default Controller class.
    '''
    def _setup_parser(self):
        '''
        Set up the parser object.
        '''
        parser = argparse.ArgumentParser(description='Album list')
        parser.add_argument('--mediapath', help='path of media repository')
        parser.add_argument('--random', type=int,
                            help='print X random entries')
        parser.add_argument('--group',
                            choices=['none', 'alphabetical',
                                     'decade',
                                     'genre'],
                            help='Album grouping',
                            default='none')
        parser.add_argument('--sort',
                            choices=['title', 'year', 'runtime'],
                            help='Album sorting',
                            default='title')
        parser.add_argument('--pagebreaks',
                            action=argparse.BooleanOptionalAction,
                            help='Page breaks on output')
        parser.add_argument('--stats',
                            action=argparse.BooleanOptionalAction,
                            help='Report statistics')
        return parser


class MovieShowReport(MovieReport):
    '''
    Subclass of MovieReport class.
    '''
    def __init__(self):
        super().__init__()
        self.pagebreaks = False

    def params_from_controller(self, in_controller):
        super().params_from_controller(in_controller)
        if in_controller.args.pagebreaks:
            self.pagebreaks = True

    def report(self, grouping=None, sorting=None, sample=0, stats=False):
        '''
        1. Output the headers.
        2. Count the batches
        '''
        if grouping:
            self.group = grouping
        if sorting:
            self.asort = sorting
        if self.sample == 0:
            self.sample = max(sample, 0)
        if stats:
            self.stats = stats
        out = ''
        if self.organizer:
            out += self.report_body()
        if self.stats:
            out += self._stats()
        return out

    def report_body(self):
        out = ''
        self.organizer.set_grouping(self.group)
        if self.sample > 0:
            batches = self.organizer.create_batches(self.group, self.sample)
        else:
            batches = self.organizer.create_batches(self.group)
        if len(batches) == 1:
            out += self._out_batch(batches[0], self.asort)
        else:
            for batch_i in sorted(batches):
                out += f"  -- {batch_i.header} ({len(batch_i.entries)}) --\n"
                out += self._out_batch(batch_i, self.asort)
            out = out[:-1]
        return out

    def _stats(self):
        '''
        Output the basic stats on the number of movies,
        and the number of movies in the sample set
        (if one is defined)
        '''
        all_c = 0
        wrk_c = 0
        if self.organizer:
            all_c = len(self.organizer.entries)
            wrk_c = len(self.organizer.working)
        out = "\n    ---- Movie Statistics ----\n\n"
        out += f"  {'Movie count':12s} : {all_c:5d}\n"
        if wrk_c < all_c:
            wrk_p = float(wrk_c) / all_c * 100
            out += f"  {'Sample count':12s} : {wrk_c:5d} ({wrk_p:5.2f}%)\n"
        return out

    def _out_batch(self, batch, sort_field=1):
        '''
        Generate the output for a single batch.
        '''
        out = ''
        order_list = batch.index_by(sort_field)
        for movie in order_list:
            out += str(media.fmt.text.movie.Brief(movie.movie))
            out += "\n"
            if self.pagebreaks:
                out += chr(12)
        return out


def main_cli():
    '''
    Entry point for script.

    Steps:
      1. Argument parsing/environment setup
      2. Repository loading
      3. Batch processing
      4. Listing
    '''
    controller = ShowController()
    controller.setup()

    repo = Repo(controller.mediapath)
    repo.scan()
    repo.load(FilenameMatches.All_Media)
    movies = repo.get_movies()
    movie_report = MovieShowReport()
    movie_report.set_movies(movies)
    movie_report.params_from_controller(controller)
    print(movie_report.report())


if __name__ == '__main__':
    main_cli()



#!/usr/bin/env python

#
# Copyright 2023 Chris Josephes
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#

'''
media.tools.meta.authorship module
'''

import datetime
import argparse
from media.data.media.meta.authorship import (
        AuthorshipRecord, Author, Catalog, CreationRecord)


if __name__ == '__main__':
    parser = argparse.ArgumentParser(
            description='Generate authorship record for files.')
    parser.add_argument('--author', help='Author of record')
    parser.add_argument('--email', help='Author email address')
    parser.add_argument('--title', help='Title of record')
    args = parser.parse_args()

    author_record = AuthorshipRecord()
    catalog = Catalog()
    author_info = Author()
    if args.title:
        author_record.title = args.title
    if args.author or args.email:
        if args.author:
            author_info.name = args.author
        if args.email:
            author_info.email = args.email
        catalog.authors.append(author_info)
    if catalog:
        author_record.catalog = catalog

    creation = CreationRecord()
    creation.tstamp = datetime.date.today()
    if author_info.name:
        creation.authors.append(author_info)
    author_record.changelog.append(creation)

    print(author_record.to_xml())



#!/usr/bin/env python

#
# Copyright 2025 Chris Josephes
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#

'''
Abstract formatter class that all sublclasses inherit.
'''

# pylint: disable=R0903

import importlib


class AbstractFormatter():
    '''
    Abstract formatter class.
    '''

    structure_matrix = {}

    def __init__(self):
        self.rend_obj_pool = {}
        self.rend_count = {}

    def render(self, in_structure_obj):
        '''
        Launch a formatter object and return the output
        stream.
        '''
        output = ''
        rend_obj = self.get_renderer_for_structure(in_structure_obj)
        if rend_obj:
            output = rend_obj.render(in_structure_obj)
            self._counter_tally(rend_obj)
        return output

    def get_renderer_for_structure(self, in_object):
        '''
        Return a rendering object spefici to the structure object.
        '''
        struct_class = in_object.__class__
        if struct_class in self.structure_matrix:
            rend_class = self.structure_matrix[struct_class]
            if rend_class not in self.rend_obj_pool:
                rend_module = rend_class.__module__
                im_module = importlib.import_module(rend_module)
                rend_obj = getattr(im_module, rend_class.__name__)()
                self.rend_obj_pool[rend_class] = rend_obj
            return self.rend_obj_pool[rend_class]
        return None

    def _counter_tally(self, in_rend_obj):
        rend_class = in_rend_obj.__class__
        if rend_class in self.rend_count:
            self.rend_count[rend_class] += 1
        else:
            self.rend_count[rend_class] = 1



#!/usr/bin/env python

#
# Copyright 2023 Chris Josephes
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#

'''
Main validator class
'''
# from media.validation.tests.media.internal import MediaTests
from media.validation.core.status import Tray
from media.validation.suites.default import DefaultValidatorSuite


class Validator():
    '''
    Main validator class that loads all media objects,
    and runs all tests.
    '''
    def __init__(self):
        self.petre = []
        self.suite = None

    def set_suite(self, in_suite=None):
        '''
        Set the main test class used in the tests.
        '''
        if not in_suite:
            self.suite = DefaultValidatorSuite()
        else:
            self.suite = in_suite

    def load_media(self, in_media, in_level=5):
        '''
        Load all of the media classes to be tested.
        '''
        for media in in_media:
            self.petre.append(Tray(media, in_level))

    def run_tests(self):
        '''
        Run tests against media objects, then
        calculate final scores.
        '''
        # results = []
        for dish in self.petre:
            self.suite.run(dish, dish.level)
        for dish in self.petre:
            dish.status.calculate()
        return self.petre



#!/usr/bin/env python

#
# Copyright 2025 Chris Josephes
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#

'''
Run simple validation tests against the movies, report any incomplete data.
'''

# pylint: disable=R0801
# pylint: disable=too-few-public-methods

import argparse
import os
from datetime import datetime
from media.data.media.medium import get_medium_type
from media.general.sorting.organizer import Organizer
from media.tools.common import load_media_dev

from media.validation.core.validator import Validator
from media.validation.core.status import Tally


def filter_output_set(in_results, in_args):
    '''
    Output a random sample of records.
    '''
    outset = []
    for result in in_results:
        if in_args.filter == 'none':
            outset.append(result)
        elif in_args.filter == 'passing':
            if result.has_passed():
                outset.append(result)
        else:
            if not result.has_passed():
                outset.append(result)
    if in_args.random:
        return Organizer.get_random_sample(outset, in_args.random)
    return outset


def output_list(filtered_list):
    '''
    Output a simple list of results.
    '''
    print(Tally.header(), end='')
    for sample_i in filtered_list:
        mtype = get_medium_type(sample_i.media.medium)
        status_o = sample_i.status
        print(f"{sample_i.media!s:40.39s} {mtype:10s} {status_o.tally!s}")


def output_details(filtered_list):
    '''
    Output the full status report of every piece of media.
    '''
    for sample_i in filtered_list:
        print(sample_i.status.report())


def output_timestamp(in_start_time):
    '''
    Output program run timestamp data.
    '''
    end_time = datetime.now()
    out = f"\n {'Program Start Time'} : {in_start_time}\n" + \
          f"   {'Program End Time'} : {end_time}\n" + \
          f"         {'Duration'}   : {end_time - in_start_time}\n"
    return out


def single_column_stats(in_results):
    '''
    Output a single column of stats.
    '''
    output = "\n"
    all_passed = 0
    all_failed = 0
    all_total = len(in_results)
    for result in in_results:
        if result.has_passed():
            all_passed += 1
        else:
            all_failed += 1
    all_perc = all_passed / all_total * 100
    output += f"{' ':20s} {'Entire Set'}\n" + \
              f"{' ':20s} {'-'*10}\n" + \
              f"{'Total Media:':20s} {all_total:10d}\n" + \
              f"{'Perfect:':20s} {all_passed:10d}\n" + \
              f"{'Failed:':20s} {all_failed:10d}\n\n" + \
              f"{'Overall Percentage:':20s} {all_perc:9.2f}%\n"
    return output


def double_column_stats(in_results, in_filtered_results):
    '''
    Output two columns of stats, comparing all results
    and the filtered results.
    '''
    output = "\n"
    all_total = len(in_results)
    sample_total = len(in_filtered_results)
    all_passed = 0
    all_failed = 0
    sample_passed = 0
    sample_failed = 0
    for result in in_results:
        if result.has_passed():
            all_passed += 1
        else:
            all_failed += 1
    for result in in_filtered_results:
        if result.has_passed():
            sample_passed += 1
        else:
            sample_failed += 1
    all_perc = all_passed / all_total * 100
    sample_perc = sample_passed / sample_total * 100
    output += f"{' ':20s} {'Entire Set'} {'Sample Set'}\n" + \
              f"{' ':20s} {'-'*10} {'-'*10}\n" + \
              f"{'Total Media:':20s} {all_total:10d} {sample_total:10d}\n" + \
              f"{'Passed:':20s} {all_passed:10d} {sample_passed:10d}\n" + \
              f"{'Failed:':20s} {all_failed:10d} {sample_failed:10d}\n\n" + \
              f"{'Overall Percentage:':20s} {all_perc:9.2f}% " +\
              f"{sample_perc:9.2f}%\n"
    return output


def output_stats(in_start_time, in_results, in_filtered):
    '''
    Output overall statistics, including time metrics.
    '''
    o_total = len(in_results)
    f_total = len(in_filtered)
    output = ''
    if f_total == o_total:
        output = single_column_stats(in_results)
    else:
        output = double_column_stats(in_results, in_filtered)
    output += output_timestamp(in_start_time)
    print(output)


if __name__ == '__main__':
    start_time = datetime.now()
    parser = argparse.ArgumentParser(description='Simple movie list.')
    parser.add_argument('--mediapath',
                        help='path of media library')
    parser.add_argument('--level',
                        type=int,
                        default=4,
                        choices=range(1, 6),
                        help='diagnostic level')
    parser.add_argument('--filter',
                        choices=['passing', 'failed', 'none'],
                        default='none',
                        help='records to output')
    parser.add_argument('--random', type=int, help='show X random entries')
    parser.add_argument('--list',
                        action=argparse.BooleanOptionalAction,
                        default=True,
                        help='list records')
    parser.add_argument('--details',
                        action=argparse.BooleanOptionalAction,
                        default=False,
                        help='show record details')
    parser.add_argument('--stats', action='store_true',
                        help='report statistics')
    args = parser.parse_args()
    mediapath = args.mediapath or os.environ['MEDIAPATH']
    if not mediapath:
        parser.print_help()
    media = load_media_dev(mediapath)
    sample = media
    validator = Validator()
    validator.set_suite()
    validator.load_media(sample, args.level)
    results = validator.run_tests()
    if args.filter:
        filtered_results = filter_output_set(results, args)
    else:
        filtered_results = results
    if args.list:
        output_list(filtered_results)
    if args.details:
        output_details(filtered_results)
    if args.stats:
        output_stats(start_time, results, filtered_results)



#!/usr/bin/env python

#
# Copyright 2025 Chris Josephes
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#

'''
Keywords module
'''

# pylint: disable=too-few-public-methods
# pylint: disable=consider-using-dict-items

from media.data.nouns import Art, Noun, PersonalName, Place, Group, Entity
from media.xml.namespaces import Namespaces


class Keywords():
    '''
    Main container object for a list of keyword objects
    Main container class is a hash table of arrays, which can get tricky.

    Keywords are either GenericKeyword objects, or ProperNoun objects.

    Every keyword has an integer relevance value to signify importance.
    The values range from 1 (most important) to 5 (least important) with
    the default value being 3.

    All keywords are kept in an array of pools (known as collections
    in the XML schema.  If no collections is defined for a keyword,
    it goes in the 'generic' pool.

    Groups are a feature of the XML schema that allow multiple
    keyword objects to share the same relevance value, or the
    same collection value.
    '''
    def __init__(self, in_kw_element):
        '''Initialize keyword bundle'''
        self.pools = {}
        if in_kw_element is not None:
            self._process(in_kw_element)

    def all(self):
        '''Return all keywords in a sorted array'''
        out = []
        for pool_n in self.pools:
            out.extend(self.pools[pool_n])
        return sorted(out)

    def pool_list(self):
        '''Returns a list of keyword pools'''
        return self.pools.keys()

    def all_by_pools(self):
        '''Returns a dictionary of keywords, ordered by the pool name'''
        out_p = {}
        for pool_n in self.pools:
            out_p[pool_n] = sorted(self.pools[pool_n])
        return out_p

    def _process(self, in_kw_element):
        for child in in_kw_element:
            tname = Namespaces.ns_strip(child.tag)
            if tname == 'generic':
                self._add_single_kw(child)
            elif tname == 'properNoun':
                self._add_proper_noun(child)
            elif tname == 'group':
                self._add_group(child)

    def _add_single_kw(self, child):
        if child.text:
            if child.text.strip() != '':
                gen_kw = GenericKeyword(child)
                self._add_to_pool(gen_kw)

    def _add_proper_noun(self, child):
        prn_kw = ProperNounKeyword(child)
        self._add_to_pool(prn_kw)

    def _add_to_pool(self, kw_object):
        pool = kw_object.pool
        if pool in self.pools:
            self.pools[pool].append(kw_object)
        else:
            self.pools[pool] = [kw_object]

    def _add_group(self, group_element):
        pool = None
        relevance = None
        if 'collection' in group_element.attrib:
            pool = group_element.attrib['collection']
        if 'relevance' in group_element.attrib:
            relevance = int(group_element.attrib['relevance'])
        for child in group_element:
            tname = Namespaces.ns_strip(child.tag)
            if tname == 'generic':
                if child.text:
                    if child.text.strip() != '':
                        gen_kw = GenericKeyword(child)
                        if pool:
                            gen_kw.pool = pool
                        if relevance:
                            gen_kw.relevance = relevance
                        self._add_to_pool(gen_kw)
            if tname == 'properNoun':
                prn_kw = ProperNounKeyword(child)
                if pool:
                    prn_kw.pool = pool
                if relevance:
                    prn_kw.relevance = relevance
                self._add_to_pool(prn_kw)


class AbstractKeyword():
    '''
    Parent class that both GenericKeyword and ProperNounKeyword inherit from
    '''
    def __init__(self, in_element):
        self.value = ''
        self.sort_value = ''
        self.type = ''
        self.relevance = 3
        self.synonym = None
        self.pool = 'generic'
        self.clarification = None
        self._process_attributes(in_element)

    def _process_attributes(self, in_element):
        if 'relevance' in in_element.attrib:
            self.relevance = int(in_element.attrib['relevance'])
        if 'collection' in in_element.attrib:
            self.pool = in_element.attrib['collection']
        if 'clarification' in in_element.attrib:
            self.clarification = in_element.attrib['clarification']
        if 'synonym' in in_element.attrib:
            self.synonym = in_element.attrib['synonym']

    def detail(self):
        '''
        Return detailed data on the keyword
        '''
        return self.type + '/' + self.value

    def __hash__(self):
        return hash(self.value)

    def __lt__(self, other):
        '''
        Sorting for all keywords is tricier, becase
        the relevance value has to be accounted for. If
        the keyword "banana" has a lower relevance value than
        the keyword "apple", then "banana" comes first in the
        sort evaluation.

        All subclasses of AbstractKeyword should inherit
        this mechanism, and the subclasses should be
        able to compare themselves to one another.
        '''
        if self.relevance == other.relevance:
            return self.sort_value < other.sort_value
        return self.relevance < other.relevance

    def __gt__(self, other):
        if self.relevance == other.relevance:
            return self.sort_value > other.sort_value
        return self.relevance > other.relevance

    def __eq__(self, other):
        if self.relevance == other.relevance:
            return self.sort_value == other.sort_value
        return False


class GenericKeyword(AbstractKeyword):
    '''
    A generic keyword is a simple string that doesn't have any
    unique traits, compared to a ProperNounKeyword.  It's
    value could represent a common noun, or a verb, or
    almost anything.

    It has values for relevance, a synonym string, a
    clarification string, and a pool assignments.

    '''
    def __init__(self, in_element):
        super().__init__(in_element)
        self.value = in_element.text.strip()
        self.sort_value = self.value.casefold()
        self.type = 'generic'

    def __str__(self):
        return self.value


class ProperNounKeyword(AbstractKeyword):
    '''
    Proper name keyword, which is tricker, because there is
    an embedded pproper noun inside the element that has to
    be accounted for, alongside the attributes of the
    keyword properNoun element.
    '''
    def __init__(self, in_element):
        super().__init__(in_element)
        self.type = 'properNoun'
        self._process(in_element)
        self.sort_value = str(self.value.sort_value).casefold()

    def _process(self, in_element):
        if in_element is not None:
            child = in_element[0]
            tagname = Namespaces.ns_strip(child.tag)
            self.tagtype = tagname
            if tagname in ['thing', 'event']:
                self.value = Noun(child)
            elif tagname == 'person':
                self.value = PersonalName(child)
            elif tagname == 'place':
                self.value = Place(child)
            elif tagname == 'group':
                self.value = Group(child)
            elif tagname == 'entity':
                self.value = Entity(child)
            elif tagname == 'art':
                self.value = Art(child)

    def detail(self):
        return self.type + '/' + self.tagtype + '/' + str(self.value)

    def __str__(self):
        return str(self.value)



#!/usr/bin/env python

#
# Copyright 2024 Chris Josephes
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#

'''
Album Catalog objects
'''

# pylint: disable=too-few-public-methods

from media.data.media.contents.generic.catalog import AbstractCatalog
from media.data.nouns import AbstractNoun, noun_dispatcher
from media.xml.namespaces import Namespaces


class AlbumCatalog(AbstractCatalog):
    '''
    Subclass for handling catalog data specific
    to a music album.
    '''
    def __init__(self, in_element):
        super().__init__()
        self.artists = []
        self.composers = []
        if in_element is not None:
            self._process(in_element)

    def _process(self, in_element):
        super()._process(in_element)
        for child in (in_element):
            e_name = Namespaces.ns_strip(child.tag)
            if e_name == 'artists':
                self._process_artists(child)

    def _process_artists(self, in_element):
        for child in in_element:
            e_name = Namespaces.ns_strip(child.tag)
            if e_name == 'artist':
                self.artists.append(noun_dispatcher(child))
            elif e_name == 'variousArtists':
                self.artists.append(VariousArtists())
                break


class VariousArtists(AbstractNoun):
    '''
    Representing a collection of multiple
    artists, either individuals or groups.
    '''
    def __init__(self):
        super().__init__()
        self.value = 'Various Artists'
        self.sort_value = 'zzz_various_artists'



#!/usr/bin/env python

#
# Copyright 2025 Chris Josephes
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#

'''The main media classes'''

# pylint: disable=too-few-public-methods

from media.general.stringtools import build_sort_string
from media.xml.namespaces import Namespaces
import media.data.media.library
import media.data.media.medium
import media.data.media.contents.audio.album
import media.data.media.contents.movie
import media.data.media.meta.authorship as MA


class Media():
    '''Representation of a physical thing that holds content.'''
    def __init__(self, in_chunk):
        self.author_record = None
        self.title = None
        self.library = None
        self.medium = None
        self.contents = []
        self.unique_key = ""
        self._process(in_chunk)

    def _process(self, in_chunk):
        '''Read the passed elemennt and load the interpret the data'''
        for child in in_chunk:
            if child.tag == Namespaces.nsf('authorship') + 'authorshipRecord':
                self.author_record = MA.AuthorshipRecord(child)
            if child.tag == Namespaces.nsf('media') + 'title':
                self.title = Title(child)
            elif child.tag == Namespaces.nsf('media') + 'library':
                self.library = media.data.media.library.Library(child)
            elif child.tag == Namespaces.nsf('media') + 'medium':
                self.medium = media.data.media.medium.Medium(child)
            elif child.tag == Namespaces.nsf('media') + 'contents':
                self._load_contents(child)
        if self.title is not None:
            self._build_unique_key()

    def _build_unique_key(self):
        self.unique_key = self.title.sort_title

    def _load_contents(self, in_chunk):
        '''Build an array to hold the contents in the media'''
        for element in in_chunk:
            if element.tag == Namespaces.nsf('movie') + 'movie':
                self.contents.append(
                        media.data.media.contents.movie.Movie(element))
            if element.tag == Namespaces.nsf('audio') + 'album':
                self.contents.append(
                        media.data.media.contents.audio.album.Album(element))

    def __str__(self):
        return f"{self.title!s}"


class Title():
    '''Representative of the title of a piece of media'''
    def __init__(self, in_element):
        self.title = None
        self.edition = None
        if in_element is not None:
            self._load_title(in_element)

    def _load_title(self, in_element):
        '''Compose the title based on the passed elements'''
        for child in in_element:
            if child.tag == Namespaces.nsf('media') + 'main':
                self.title = child.text
            if child.tag == Namespaces.nsf('media') + 'edition':
                self.edition = child.text
        self.sort_title = build_sort_string(self.title)

    def __str__(self):
        '''Return the title of the media'''
        if self.edition:
            return f"{self.title} ({self.edition})"
        return f"{self.title}"



#!/usr/bin/env python

#
# Copyright 2025 Chris Josephes
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#

'''
Shared classes and subroutines for the list and show
modules.
'''


# pylint: disable=too-few-public-methods, R0801


import argparse
import os
import media.fmt.text.movie
from media.general.sorting.organizer import Organizer
from media.general.sorting.batch import Batch

CliGroupingOptions = {
        'none': Organizer.G_NONE,
        'alphabetical': Organizer.G_ANY_ALPHA,
        'decade': Organizer.G_ANY_DECADE,
        'genre': Organizer.G_ANY_GENRE,
}


CliSortOptions = {
        'title': Batch.S_TITLE,
        'year': Batch.S_YEAR,
        'runtime': Batch.S_RUNTIME,
}


class Controller():
    '''
    The Controller class manages aspects of the command
    execution.  It processes all of the CLI arguments,
    and tests file paths to make sure they exist.
    '''
    def __init__(self):
        self.args = None
        self.mediapath = None
        self.group = Organizer.G_NONE
        self.sort = Batch.S_TITLE

    def setup(self):
        '''
        Initialize the command line parser, and make sure
        the mediapath exists.
        '''
        parser = self._setup_parser()
        self.args = parser.parse_args()
        self._convert_args()
        self._determine_path()

    def _setup_parser(self):
        '''
        Set up the parser object.
        '''
        parser = argparse.ArgumentParser(description='Movie list')
        parser.add_argument('--mediapath', help='path of media repository')
        parser.add_argument('--random', type=int,
                            help='print X random entries')
        parser.add_argument('--group',
                            choices=['none', 'alphabetical',
                                     'decade',
                                     'genre'],
                            help='Movie grouping',
                            default='none')
        parser.add_argument('--sort',
                            choices=['title', 'year', 'runtime'],
                            help='Movie sorting',
                            default='title')
        parser.add_argument('--stats',
                            action=argparse.BooleanOptionalAction,
                            help='Report statistics')
        return parser

    def _convert_args(self):
        if self.args.group in CliGroupingOptions:
            self.group = CliGroupingOptions[self.args.group]
        if self.args.sort in CliSortOptions:
            self.sort = CliSortOptions[self.args.sort]

    def _determine_path(self):
        '''
        Verify that the media path with the XML files exists.
        '''
        mediapath = self.args.mediapath
        if not mediapath:
            if 'MEDIAPATH' in os.environ:
                mediapath = os.environ['MEDIAPATH']
        self.mediapath = mediapath


class MovieReport():
    '''
    Class that does all the listing work.

    Accepts a list of movies as input,
    and generates a report output as a string.

    Should also be directly callable from
    an interactive Python prompt.
    '''
    def __init__(self):
        self.organizer = None
        self.movies = []
        self.sample = 0
        self.group = Organizer.G_ANY_ALPHA
        self.asort = Batch.S_TITLE
        self.stats = False

    def set_movies(self, in_movies, in_controller=None):
        '''
        Pass all the Movie content objects that were
        found in the repository.
        '''
        if len(in_movies) > 0:
            self.movies = in_movies
            self.organizer = Organizer(in_movies)
            if in_controller:
                self.params_from_controller(in_controller)

    def params_from_controller(self, in_controller):
        '''
        Get the parameters from the controller object.
        '''
        self.group = CliGroupingOptions[in_controller.args.group]
        self.asort = CliSortOptions[in_controller.args.sort]
        if in_controller.args.random:
            self.sample = in_controller.args.random
        if in_controller.args.stats:
            self.stats = True

    def report(self, grouping=None, sorting=None, sample=0, stats=False):
        '''
        1. Output the headers.
        2. Count the batches
        '''
        if grouping:
            self.group = grouping
        if sorting:
            self.asort = sorting
        if self.sample == 0:
            self.sample = max(sample, 0)
        if stats:
            self.stats = stats
        out = ''
        out += media.fmt.text.movie.OneLiner.header_fields()
        if self.organizer:
            out += self.report_body()
        if self.stats:
            out += self._stats()
        return out

    def report_body(self):
        '''
        Body of the report.
        '''
        out = ''
        if self.sample > 0:
            batches = self.organizer.create_batches(self.group, self.sample)
        else:
            batches = self.organizer.create_batches(self.group)
        if len(batches) == 1:
            out += self._out_batch(batches[0], self.asort)
        else:
            for batch_i in sorted(batches):
                out += f"  -- {batch_i.header} ({len(batch_i.entries)}) --\n"
                out += self._out_batch(batch_i, self.asort)
        out = out[:-1]
        out += media.fmt.text.movie.OneLiner.header_line()
        return out

    def _stats(self):
        '''
        Output the basic stats on the number of movies,
        and the number of movies in the sample set
        (if one is defined)
        '''
        all_c = 0
        wrk_c = 0
        if self.organizer:
            all_c = len(self.organizer.entries)
            wrk_c = len(self.organizer.working)
        out = f"\n  {'Movie count':12s} : {all_c:5d}\n"
        if wrk_c < all_c:
            wrk_p = float(wrk_c) / all_c * 100
            out += f"  {'Sample count':12s} : {wrk_c:5d} ({wrk_p:5.2f}%)\n"
        return out

    def _out_batch(self, batch, sort_field=Batch.S_TITLE):
        '''
        Generate the output for a single batch.
        '''
        out = ''
        order_list = batch.index_by(sort_field)
        for movie in order_list:
            out += str(media.fmt.text.movie.OneLiner(movie.movie))
        out += "\n"
        return out



#!/usr/bin/env python

#
# Copyright 2022 Chris Josephes
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#

'''
Objects for representation of dates, either exact, or a date range.
'''

# pylint: disable=too-few-public-methods


class AbstractDate():
    '''
    Abstract class which has data shared by both
    child classes.
    '''
    def __init__(self):
        self.date = None


class ExactDate(AbstractDate):
    '''
    Representing an exact calendar date.
    '''
    def __init__(self, in_element):
        super().__init__()
        self.date = in_element.text

    def __str__(self):
        return f"{self.date}"


class RangeDate(AbstractDate):
    '''
    Representation for a non-specific date, where both an estimated start
    and an estimated end is known.  For example, if something was acquired
    in 2014, the start value would be 2014-01-01, and the end value would be
    P1Y, to allow coverage for every day in 2014.
    '''
    def __init__(self, in_element_1, in_element_2):
        super().__init__()
        self.date = in_element_1.text
        self.end = in_element_2.text

    def __str__(self):
        return f"{self.date} -> {self.end}"



#!/usr/bin/env python

#
# Copyright 2022 Chris Josephes
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#

'''The product identification classes'''

# pylint: disable=too-few-public-methods

from media.xml.functions import xs_bool
from media.xml.namespaces import Namespaces


class ProductId():
    '''Structure for all elements pertaining to product identification.'''
    def __init__(self, in_chunk):
        self.barcodes = []
        self.skus = []
        self.others = []
        self._process(in_chunk)

    def _process(self, in_chunk):
        for child in in_chunk:
            if child.tag == Namespaces.nsf('media') + 'barcode':
                self.barcodes.append(Barcode(child))
            elif child.tag == Namespaces.nsf('media') + 'sku':
                self.skus.append(SKU(child))
            elif child.tag == Namespaces.nsf('media') + 'other':
                self.others.append(OtherId(child))


class Barcode():
    '''Representation for a barcode on a piece of physical media.'''
    def __init__(self, in_element):
        self.value = None
        self.type = None
        self.scanlines = True
        if in_element.text:
            chunk = in_element.text.strip()
            if chunk != '':
                self.value = chunk
        if 'type' in in_element.attrib:
            self.type = in_element.attrib['type'].strip()
        if 'scanlines' in in_element.attrib:
            self.scanlines = xs_bool(in_element.attrib['scanlines'])

    def __str__(self):
        return f"{self.value} ({self.type})"


class SKU():
    '''Representation of a Retailer SKU for a physical piece of media.'''
    def __init__(self, in_element):
        self.value = None
        self.retailer = ''
        self.type = None
        if in_element.text:
            chunk = in_element.text.strip()
            if chunk != '':
                self.value = chunk
        if 'retailer' in in_element.attrib:
            self.retailer = in_element.attrib['retailer'].strip()
        if 'type' in in_element.attrib:
            self.type = in_element.attrib['type'].strip()

    def __str__(self):
        return f"{self.value} ({self.retailer})"


class OtherId():
    '''Other ID values, like a "Proof Of Purchase".'''
    def __init__(self, in_element):
        self.name = None
        self.values = []
        for child in in_element:
            if child.tag == Namespaces.nsf('media') + 'name':
                self.name = child.text.strip()
            if child.tag == Namespaces.nsf('media') + 'value':
                self.values.append(child.text.strip())



#!/usr/bin/env python

#
# Copyright 2024 Chris Josephes
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#

'''
Album Catalog objects
'''

# pylint: disable=too-few-public-methods

from datetime import timedelta


class AlbumTechnical():
    '''
    Subclass for handling catalog data specific
    to a music album.
    '''
    def __init__(self, in_album):
        self.runtime = timedelta(0)
        self._build_total_runtime(in_album)

    def _build_total_runtime(self, in_album):
        for elem in in_album.elements:
            if elem.technical is not None:
                if elem.technical.runtime is not None:
                    rtime = elem.technical.runtime.overall
                    self.runtime += rtime



#!/usr/bin/env python

#
# Copyright 2024 Chris Josephes
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#

'''
Song object structure
'''

# pylint: disable=too-few-public-methods
# pylint: disable=R0801

from media.data.media.contents.genericv.technical import process_iso_duration
from media.data.media.contents.audio.elements import (
        AbstractElement, ElementTechnical
        )
from media.xml.namespaces import Namespaces


class Dialogue(AbstractElement):
    '''
    Object representation of a song.
    '''
    def __init__(self, in_element, in_parent):
        super().__init__()
        self.classification = None
        self.technical = None
        self.parent_o = in_parent
        self._process(in_element)

    def _process(self, in_element=None):
        super()._process(in_element)
        for child in in_element:
            e_name = Namespaces.ns_strip(child.tag)
            if e_name == 'technical':
                self.technical = DialogueTechnical(child)


class DialogueTechnical(ElementTechnical):
    '''
    Object representation of a song's technical data.
    '''
    def __init__(self, in_element):
        super().__init__(in_element)
        self.recording = None
        if in_element is not None:
            self._process(in_element)

    def _process(self, in_element):
        super()._process(in_element)
        for child in in_element:
            e_name = Namespaces.ns_strip(child.tag)
            if e_name == 'runtime':
                self.runtime = DialogueTechnicalRuntime(child)


class DialogueTechnicalRuntime():
    '''
    Song runtime data.
    '''
    def __init__(self, in_element):
        self.overall = None
        if in_element is not None:
            self._process(in_element)

    def _process(self, in_element):
        for child in in_element:
            e_name = Namespaces.ns_strip(child.tag)
            if e_name == 'overall':
                dur_string = child.text
                dur_value = process_iso_duration(dur_string)
                self.overall = dur_value



#!/usr/bin/env python

#
# Copyright 2023 Chris Josephes
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#


'''
generic validation module
'''

# pylint: disable=R0801
# pylint: disable=W0613

from datetime import timedelta
from media.validation.core.faults import Fault, FaultLevel
from media.validation.core.status import StatusCode, TestResult
from media.validation.core.tests import AbstractValidator

DEFMAXSCORE = 5


class GenericVisualTechnicalValidator(AbstractValidator):
    '''
    Generic test for the technical element
    on all visual content (movies, television)

    Specific tests for movies or television
    should go in their respective classes.
    '''
    def __init__(self):
        super().__init__()
        self.load_objects()

    def load_objects(self):
        '''
        Load additional test classes.
        '''
        self.objects.append(GenericVisualRuntimeValidator())

    def run_standard_tests(self, in_dish, in_level=5):
        '''
        Run all tests.
        '''
        self.local_tests = [self.test_runtime_presence]
        for ltst in self.local_tests:
            ltst(in_dish)
        for tobj in self.objects:
            tobj.run_standard_tests(in_dish, in_level)

    def test_runtime_presence(self, in_dish):
        '''
        Test for the presence of the runtime element data.
        '''
        name = 'genericv.technical.runtime'
        test_r = TestResult(name, DEFMAXSCORE)
        technical = in_dish.item.technical
        if technical is not None:
            runtime = technical.runtime
            if runtime:
                test_r.autopass()
            else:
                fault = Fault(FaultLevel.NOTICE,
                              'Technical block should have runtime element')
                test_r.result(0, StatusCode.FAIL, fault)
        else:
            test_r.skip()
        in_dish.add_result(test_r)


class GenericVisualRuntimeValidator(AbstractValidator):
    '''
    Tests against the runtime value of all A/V
    content objects.
    '''

    def run_standard_tests(self, in_dish, in_level=5):
        '''
        Run the all test methods.
        '''
        self.test_runtime_value(in_dish)

    def test_runtime_value(self, in_dish):
        '''
        Test runtime value to see if it has been
        properly set.
        '''
        name = 'genericv.technical.runtime.value'
        test_r = TestResult(name, DEFMAXSCORE)
        technical = in_dish.item.technical
        if technical is not None:
            runtime = technical.runtime
            if runtime:
                runtime_value = runtime.overall or timedelta(0)
                template_value = timedelta(seconds=1, minutes=1, hours=1)
                if runtime_value == template_value:
                    fault = Fault(FaultLevel.NOTICE,
                                  'Suspicious runtime value (1h1m1s)')
                    test_r.result(3, StatusCode.FAIL, fault)
                else:
                    test_r.autopass()
            else:
                test_r.skip()
        else:
            test_r.skip()
        in_dish.add_result(test_r)



#!/usr/bin/env python

#
# Copyright 2025 Chris Josephes
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#

'''Grouping classes go here.'''

# pylint: disable=too-few-public-methods

from media.general.sorting.batch import Batch


class Grouping():
    '''
    Abstract class for grouping operations.
    '''
    @classmethod
    def group(cls, entries):
        """
        Group incoming entries based on a certain trait.
        """
        _ = entries
        return {}


class GroupAll(Grouping):
    '''
    Return a single set with all objects.
    '''
    @classmethod
    def group(cls, entries):
        """
        Return a single Batch with every entry in it.
        """
        groups = {}
        groups['all'] = Batch()
        groups['all'].extend(entries)
        return groups


class GroupTitleAlphabetical(Grouping):
    '''
    Grouping entries based on their title.
    '''
    @classmethod
    def group(cls, entries):
        """
        Return multiple batches, grouped by the
        first letter in the title.
        """
        groups = {}
        for con_ind in entries:
            if con_ind.first_letter not in groups:
                f_letter = con_ind.first_letter.upper()
                groups[con_ind.first_letter] = Batch(con_ind.first_letter,
                                                     f_letter, con_ind)
            else:
                groups[con_ind.first_letter].append(con_ind)
        return groups


class GroupDecade(Grouping):
    '''
    Grouping entries based on the decade they were created.
    '''
    @classmethod
    def group(cls, entries):
        """
        Return multiple batches, grouped by
        the decade of the copyright year.
        """
        groups = {}
        for con_ind in entries:
            if con_ind.decade not in groups:
                f_decade = str(con_ind.decade) + "0s"
                groups[con_ind.decade] = Batch(con_ind.decade,
                                               f_decade, con_ind)
            else:
                groups[con_ind.decade].append(con_ind)
        return groups


class GroupPrimaryGenre(Grouping):
    '''
    Grouping entries based on their primary genre.
    '''
    @classmethod
    def group(cls, entries):
        """
        Return multiple batches, grouped by
        the primary genre of the content.
        """
        groups = {}
        for con_ind in entries:
            if con_ind.primary_g not in groups:
                groups[con_ind.primary_g] = Batch(con_ind.primary_g,
                                                  con_ind.primary_g, con_ind)
            else:
                groups[con_ind.primary_g].append(con_ind)
        return groups



#!/usr/bin/env python

#
# Copyright 2025 Chris Josephes
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#

'''
Module to report on all named within a movie
'''

# pylint: disable=R0801

import os
import argparse
from media.general.sorting.organizer import Organizer
from media.tools.common import load_movies


class ActorRoleMap():
    '''
    Align an actor with all of their roles
    '''
    def __init__(self, in_role, in_movie):
        self.name = in_role.actor
        self.movies = {}
        self.movies[in_movie] = in_role.portrays

    def add_movie(self, in_role, in_movie):
        '''
        Add movie object and role data to the entry.
        '''
        if in_movie in self.movies:
            self.movies[in_movie].extend(in_role.portrays)
        else:
            self.movies[in_movie] = in_role.portrays

    def movie_count(self):
        """
        Return a count of movies for a given actor.
        """
        return len(self.movies.keys())

    def get_movies(self):
        """
        Return all the movie objects for a given actor.
        """
        return self.movies.keys()

    @classmethod
    def header(cls):
        '''
        Generate a report header
        '''
        out = f"{'Actor Name':30s} {'Title':30s} {'Portrays':30s}\n" + \
              cls.header_line()
        return out

    @classmethod
    def header_line(cls):
        """
        Return a header line.
        """
        return f"{'=' * 30} {'=' * 30} {'=' * 30}\n"

    def __str__(self):
        out = ''
        name_c = 1
        for movie in sorted(self.movies):
            title_c = 1
            for portrays in self.movies[movie]:
                if name_c == 1:
                    out += f"{self.name!s:30s} {movie.title!s:30s} " + \
                           f"{portrays.formal!s:30s}\n"
                else:
                    if title_c == 1:
                        out += f"{' ':30s} {movie.title!s:30s} " + \
                               f"{portrays.formal!s:30s}\n"
                    else:
                        out += f"{' ':30s} {' ':30s} {portrays.formal!s:30s}\n"
                name_c += 1
                title_c += 1
        return out

    def __lt__(self, other):
        return self.name < other.name

    def __rt__(self, other):
        return self.name > other.name

    def __eq__(self, other):
        return self.name == other.name


def grab_cast_names(in_movies):
    '''
    Extract cast names from movies.
    '''
    ca_dict = {}
    for movie in in_movies:
        if movie.crew is not None:
            if movie.crew.cast:
                for role in movie.crew.cast.cast:
                    if role.actor in ca_dict:
                        ca_dict[role.actor].add_movie(role, movie)
                    else:
                        ca_dict[role.actor] = ActorRoleMap(role, movie)
    return list(ca_dict.values())


def find_empty_movies(in_movies):
    """
    Identify movies that have no cast.
    """
    ca_list = []
    for movie in in_movies:
        if movie.crew:
            if not movie.crew.cast:
                ca_list.append(movie)
        else:
            ca_list.append(movie)
    return ca_list


def list_names(in_names):
    '''
    Output all of the passed names.
    '''
    print(ActorRoleMap.header(), end='')
    for nm_i in sorted(in_names):
        print(nm_i, end='')
    print(ActorRoleMap.header_line(), end='')


def report_stats(in_names, in_sample):
    """
    Report stats on data, whether it's all actors
    or a random sample.
    """
    all_n = stats_compile(in_names)
    partial_n = None
    if len(in_sample) < len(in_names):
        partial_n = stats_compile(in_sample)
    print()
    if partial_n:
        report_stats_sample(all_n, partial_n)
    else:
        report_stats_all(all_n)


def report_stats_all(all_n):
    """
    Formatted output when no random sub-sample
    of actor data was requested.
    """
    print(f"{' ':29s} {'Entire Set'}")
    print(f"{' ':29s} {'-' * 10}")
    print(f"{'Actor count:':29s} {all_n['names']:10d}")
    print(f"{'Movie count:':29s} {all_n['movies']:10d}")
    print(f"{'Average movies per actor:':29s} {all_n['act_mov_avg']:10.2f}")


def report_stats_sample(all_n, partial_n):
    """
    Formatted output when a ransom sub-sample
    of actor data was requested.
    """
    print(f"{' ':29s} {'Entire Set'} {'Sample Set'} {'Percentage'}")
    print(f"{' ':29s} {'-' * 10} {'-' * 10} {'-' * 10}")
    n_p = float(partial_n['names']) / all_n['names'] * 100
    m_p = float(partial_n['movies']) / all_n['movies'] * 100
    print(f"{'Actor count:':29s} " +
          f"{all_n['names']:>10d} " +
          f"{partial_n['names']:>10d} {n_p:9.2f}%")
    print(f"{'Movie count:':29s} " +
          f"{all_n['movies']:>10d} " +
          f"{partial_n['movies']:>10d} {m_p:9.2f}%")
    print(f"{'Average movies per actor:':29s} " +
          f"{all_n['act_mov_avg']:10.2f} " +
          f"{partial_n['act_mov_avg']:10.2f}")


def stats_compile(in_names):
    """
    Basic statistics on actors and films.
    A total count of the number of actors.
    A total count of the number of movies with actors.
    An average of how many acting jobs fo0r each
    actor given the sample of movies.
    """
    out_d = {}
    out_d['names'] = len(in_names)
    movies_from_actors = {}
    for act_i in in_names:
        for movie_i in act_i.movies:
            if movie_i in movies_from_actors:
                movies_from_actors[movie_i] += 1
            else:
                movies_from_actors[movie_i] = 1
    out_d['movies'] = len(movies_from_actors.keys())
    act_mov_tot = 0
    for act_i in in_names:
        act_mov_tot += act_i.movie_count()
    out_d['act_mov_avg'] = float(act_mov_tot) / out_d['names']
    return out_d


if __name__ == '__main__':
    parser = argparse.ArgumentParser(description='Simple movie list.')
    parser.add_argument('--mediapath', help='path of media library')
    parser.add_argument('--random', type=int, help='show X names')
    parser.add_argument('--stats', action=argparse.BooleanOptionalAction,
                        help='Report statistics')
    parser.add_argument('--report-empty',
                        action=argparse.BooleanOptionalAction,
                        dest='report_empty',
                        help='Report movies with no cast')
    args = parser.parse_args()
    mediapath = args.mediapath or os.environ['MEDIAPATH']
    if not mediapath:
        parser.print_help()
    all_movies = load_movies(mediapath)
    names = grab_cast_names(all_movies)
    empty = find_empty_movies(all_movies)
    if len(empty) > 0 and args.report_empty:
        print(f"{len(empty)} movies have no cast")
        for mi in empty:
            print(f"{str(mi.title)}")
    if args.random:
        rand_limit = args.random
        sample_names = Organizer.get_random_sample(names, rand_limit)
    else:
        sample_names = names
    list_names(sample_names)
    if args.stats:
        report_stats(names, sample_names)



#!/usr/bin/env python

#
# Copyright 2023 Chris Josephes
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#


'''
Comparison classes that check two movies against each other.
'''

# pylint: disable=too-few-public-methods


from media.data.media.contents.generic.keywords import ProperNounKeyword


class MovieComparator():
    '''
    Loads all films, and creates individual comparison
    operation objects for each film, allowing each
    film to maintain a separate object for scoring
    data and comparison operations.
    '''
    def __init__(self):
        self.data = []
        self.results = []

    def load_data(self, in_list):
        '''
        Load all films into the object array.
        '''
        self.data.extend(in_list)

    def compare(self):
        '''
        Build the comparator objects, and then
        have very object run the comparison
        operation against all of the films.
        '''
        for film in self.data:
            self.results.append(MovieComparison(film))
        for film_compare in self.results:
            for film in self.data:
                film_compare.compare_against(film)

    def report(self):
        '''
        Generate output reports for a given film.
        '''
        for result in self.results:
            result.report()


class MovieComparison():
    '''
    Object class for running comparison operations of
    any film against an internally stored film.
    '''
    def __init__(self, in_film):
        self.film = in_film
        self.matches = {}

    def compare_against(self, in_film):
        '''
        Compare the passed film against the film stored in the object.

        Do not run if the film passed is identical to the interal film.

        All comparison operations are literally just tests between
        'self' and 'other', similar to object comparator methods.
        '''
        if in_film == self.film:
            return
        self.compare_director(in_film)
        self.compare_writer(in_film)
        self.compare_cinemaphotographer(in_film)
        self.compare_editor(in_film)
        self.compare_cast(in_film)
        self.compare_keywords(in_film)

    def add_match(self, in_film, in_trait):
        '''
        If a match is identified, add it to the internal data structure.
        '''
        other_f = in_film
        if other_f not in self.matches:
            self.matches[other_f] = MovieMatch(other_f, in_trait)
        else:
            self.matches[other_f].add_trait(in_trait)

    def compare_director(self, in_film):
        '''
        See if the other film has identical directors.
        '''
        for director in in_film.crew.directors:
            if director in self.film.crew.directors:
                self.add_match(in_film, MatchTrait('Director', director, 10))

    def compare_writer(self, in_film):
        '''
        See if the other film has identical writers.
        '''
        for writer in in_film.crew.writers:
            if writer in self.film.crew.writers:
                self.add_match(in_film, MatchTrait('Writer', writer, 8))

    def compare_cinemaphotographer(self, in_film):
        '''
        See if the other film has identical cinemaphotographers.
        '''
        for cinema in in_film.crew.cinemap:
            if cinema in self.film.crew.cinemap:
                trait = MatchTrait('Cinemaphotographer', cinema, 7)
                self.add_match(in_film, trait)

    def compare_editor(self, in_film):
        '''
        See if the other film has similar directors.
        '''
        for editor in in_film.crew.editors:
            if editor in self.film.crew.editors:
                self.add_match(in_film, MatchTrait('Editor', editor, 6))

    def compare_cast(self, in_film):
        '''
        See if the other filn has similar actors.
        '''
        in_cast = in_film.crew.cast
        my_cast = self.film.crew.cast
        if in_cast and my_cast:
            for role in in_cast.cast:
                other_actor = role.actor
                for mrole in my_cast.cast:
                    m_actor = mrole.actor
                    if other_actor == m_actor:
                        self.add_match(in_film, MatchTrait('Cast', m_actor, 5))

    def compare_keywords(self, in_film):
        '''
        See if the other film has identical keywords.
        '''
        other_keywords = in_film.story.keywords
        keywords = self.film.story.keywords
        if other_keywords and keywords:
            for keyword in other_keywords.all():
                if keyword in keywords.all():
                    if isinstance(keyword, ProperNounKeyword):
                        kw_score = 5 + keyword.relevance
                    else:
                        kw_score = 2 + keyword.relevance
                    trait = MatchTrait('Keyword', keyword, kw_score)
                    self.add_match(in_film, trait)

    def header(self):
        '''
        Report header.
        '''
        out = f"\nSource Movie: {self.film.catalog_title()}\n" +\
              f"{'='*89}\n" +\
              f"{'Movie':35s} {'Matches':7s} " +\
              f"{'Score':5s} {'Trait':18s} {'Value':s}\n" +\
              f"{'-'*35} {'-'*7} {'-'*5} {'-'*18} {'-'*20}\n"
        return out

    def report(self):
        '''
        Generate output of comparison results.
        '''
        print(self.header(), end='')
        for other_f in sorted(self.matches,
                              key=lambda x: self.matches[x].points,
                              reverse=True):
            print(str(self.matches[other_f]), end='')


class MovieMatch():
    '''
    Object to keep track of identified matches for a given film.
    Keeps a copy of the data that matches, and a passed scoring
    value identifying the quality of the match.
    '''
    def __init__(self, in_film, in_trait):
        self.film = in_film
        self.traits = [in_trait]
        self.points = in_trait.score

    def add_trait(self, in_trait):
        '''
        If another match has been identified for the same film,
        then record it.
        '''
        self.traits.append(in_trait)
        self.points += in_trait.score

    def __str__(self):
        out = ""
        count = 1
        trait_count = len(self.traits)
        for trt in self.traits:
            if count == 1:
                out += f"{self.film.catalog_title():34.34s}  " +\
                       f"{trait_count:7d} {self.points:5d} " +\
                       f"{trt}\n"
            else:
                out += f"{' '*49} {trt}\n"
            count += 1
        return out


class MatchTrait():
    '''
    Simple data structure containing a film, the
    matching data, and the score value.
    '''
    def __init__(self, in_title, in_data, in_score=1):
        self.title = in_title
        self.data = in_data
        self.score = in_score

    def __str__(self):
        return f"{self.title:18s} {str(self.data)}"



#!/usr/bin/env python

#
# Copyright 2025 Chris Josephes
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#

'''Main module file for the Movie() and Title() objects'''

# pylint: disable=too-few-public-methods

from media.xml.namespaces import Namespaces

from media.data.media.medium.device import BaseDevice, MediumDeviceMap
from media.data.media.medium.release import (
        FormalType, Release, ReleaseException)
from media.data.media.medium.productid import ProductId
from media.data.media.medium.productspecs import ProductSpecs


class Medium():
    '''Medium object - the physical thing'''
    def __init__(self, in_element):
        self.release = None
        self.device = None
        self.new_device_used = False
        self.product_id = None
        self.product_specs = None
        self._process_xml_stream(in_element)

    def _process_xml_stream(self, in_element):
        for child in in_element:
            e_tag = Namespaces.ns_strip(child.tag)
            if e_tag in MediumDeviceMap.f_map:
                self.new_device_used = True
                self.device = BaseDevice(child)
            elif e_tag == 'release' and self.new_device_used is False:
                try:
                    self.release = Release(child)
                except ReleaseException as rel:
                    raise MediumException(rel.message) from rel
            elif e_tag == 'productId':
                self.product_id = ProductId(child)
            elif e_tag == 'productSpecs':
                self.product_specs = ProductSpecs(child)


class MediumException(Exception):
    '''
    Exceptions related to the medium object.
    '''
    def __init__(self, in_message):
        super().__init__(in_message)
        self.message = in_message

    def __str__(self):
        return self.message


def get_medium_type(in_medium):
    '''
    Simple fumction to get medium type value.
    '''
    f_type = ''
    m_type = ''
    if in_medium.device:
        f_type = in_medium.device.type_name
        m_type = MediumDeviceMap.formal_convert(f_type)
    elif in_medium.release:
        f_type = in_medium.release.type
        m_type = FormalType.formal_convert(f_type)
    else:
        m_type = 'UNKNOWN'
    return m_type



#!/usr/bin/env python

#
# Copyright 2023 Chris Josephes
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#


'''
generic validation module media/content/movie
'''

from media.validation.tests.media.contents.movie import MovieValidator
from media.data.media.contents.movie import Movie


class ValidateContentTests():
    '''
    Class to add content testing to the test lists.
    '''
    def __init__(self):
        self.tests = []
        self.setup()

    def setup(self):
        '''
        Set up content validation objects for testing.
        '''
        self.tests.append(MovieValidator())

    def run_standard_tests(self, in_dish, in_level=5):
        '''
        Run the standard tests by identifying the media type.
        '''
        for con in in_dish.status.contents:
            obj = con.item
            if issubclass(obj.__class__, Movie):
                movie_v = MovieValidator()
                movie_v.run_standard_tests(con, in_level)



#!/usr/bin/env python

#
# Copyright 2023 Chris Josephes
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#

'''Grouping classes go here.'''

# pylint: disable=too-few-public-methods

import random


class AbstractBatch():
    '''
    A class containing contect objects all tied together
    by a common grouping factor.

    The index_key value is the sorting value for comparison
    between other batch values.

    The header value is the formal string for display purposes.

    The entries list is every item in the batch.
    '''
    S_TITLE = 'sort_title'
    S_YEAR = 'year'
    S_RUNTIME = 'runtime'
    S_ARTIST = 'artists'

    def __init__(self, index_key=None, header=None, first_entry=None):
        self.index_key = index_key
        self.header = header
        self.entries = []
        if first_entry:
            self.entries = [first_entry]

    def append(self, entry):
        '''
        Add a single entry to the end
        of the interal list.
        '''
        self.entries.append(entry)

    def extend(self, entries):
        '''
        Add a bunch of entries to the end
        of the interal list.
        '''
        self.entries.extend(entries)

    def index_by(self, attribute=S_TITLE):
        '''
        Return the batch items in order based
        on an attribute value within the object.
        '''
        order_list = []
        order_list = sorted(self.entries, key=lambda x:
                            getattr(x, attribute, ''))
        return order_list

    def random_entry(self):
        '''
        Return a single random entry from the batch.
        '''
        r_num = random.randint(0, len(self.entries)-1)
        return self.entries[r_num]

    def __lt__(self, other):
        return self.index_key < other.index_key

    def __rt__(self, other):
        return self.index_key > other.index_key

    def __eq__(self, other):
        return self.index_key == other.index_key


class Batch(AbstractBatch):
    '''
    An subclass of AbstractBatch, so
    the class AbstractBatch does not need to be
    instantiated.
    '''



#!/usr/bin/env python

#
# Copyright 2025 Chris Josephes
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#

'''
Abstract object classes used by all content objects.
'''

# pylint: disable=too-few-public-methods

from media.xml.namespaces import Namespaces
from media.data.media.contents.generic.catalog import (
        Title, TitleValueException, Catalog
        )
from media.general.titletools import TitleMunger


class AbstractContent():
    '''
    Abstract class for all content, working under the
    assumption that any content object will have a
    title and a catalog.
    '''
    def __init__(self):
        self.title = None
        self.catalog = None
        self.classification = None
        self.sort_title = ""
        self.unique_key = ""

    def _process(self, in_element):
        for child in in_element:
            tagname = Namespaces.ns_strip(child.tag)
            if tagname == 'title':
                try:
                    self.title = Title(child.text)
                except TitleValueException as tve:
                    raise ContentException(tve.message) from tve
            if tagname == 'catalog':
                self.catalog = Catalog(child)

    def _post_load_process(self):
        if self.title:
            self._build_unique_key()

    def _build_unique_key(self):
        self.sort_title = TitleMunger.build_sort_cat_title(
                self.title, self.catalog)
        self.unique_key = TitleMunger.build_unique_key_string(
                self.title, self.catalog)

    def catalog_title(self):
        '''
        Return a formatted title that inclues the copyright year.
        '''
        return TitleMunger.build_catalog_title(
                self.title, self.catalog)

    def __eq__(self, other):
        return self.__class__ == other.__class__ and \
                self.unique_key == other.unique_key


class ContentException(Exception):
    '''
    Root class for all content related exceptions.
    '''
    def __init__(self, message):
        super().__init__(message)
        self.message = message

    def __str__(self):
        return self.message



#!/usr/bin/env python

#
# Copyright 2023 Chris Josephes
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#


'''
generic validation module
'''

# pylint: disable=R0801


import re
from media.validation.core.faults import Fault, FaultLevel
from media.validation.core.status import StatusCode, TestResult
from media.validation.core.tests import AbstractValidator


class GenericStoryValidator(AbstractValidator):
    '''
    Generic tests that apply to every story object.
    '''
    def __init__(self):
        super().__init__()
        self.load_objects()

    def load_objects(self):
        '''
        Load additional test objects that should be run.
        '''
        self.objects.append(GenericPlotValidator())

    def run_standard_tests(self, in_dish, in_level):
        '''
        Run all story element related tests.
        '''
        self.local_tests = [self.test_plot_presence]
        for l_tst in self.local_tests:
            l_tst(in_dish)
        for tst in self.objects:
            tst.run_standard_tests(in_dish, in_level)

    def test_plot_presence(self, in_dish):
        '''
        Confirm that a plot element is present.
        '''
        test_r = TestResult('generic.story.plot', 5)
        sty = in_dish.item.story
        if sty is not None:
            if sty.plot is not None:
                test_r.autopass()
            else:
                fault = Fault(FaultLevel.WARNING, 'No plot defined.')
                test_r.result(0, StatusCode.FAIL, fault)
        else:
            test_r.skip()
        in_dish.add_result(test_r)


class GenericPlotValidator(AbstractValidator):
    '''
    Generic tests that apply to every plot object.
    '''

    def run_standard_tests(self, in_dish, in_level=5):
        '''
        Run all plot related tests.
        '''
        if in_level <= 4:
            self.test_plot_whitespace(in_dish)
            self.test_plot_length(in_dish)

    def test_plot_whitespace(self, in_dish):
        '''
        Test plot object for leading or trailing
        whitespace.

        Technically, this test should never return a fault
        because the XML loader code does this
        automatically, since the Plot is expected to
        be a multi-line element.
        '''
        test_r = TestResult('generic.story.plot.whitesapce', 5)
        plt = in_dish.item.story.plot
        if plt is not None:
            faults = []
            p_str = str(plt)
            l_pad = re.search(r"^\s+", p_str)
            r_pad = re.search(r"\s+$", p_str)
            if l_pad is not None:
                fault = Fault(FaultLevel.WARNING, 'Leading whitespace')
                faults.append(fault)
            if r_pad is not None:
                fault = Fault(FaultLevel.WARNING, 'Trailing whitespace')
                faults.append(fault)
            if len(faults) > 0:
                test_r.result(0, StatusCode.FAIL)
                test_r.add_faults(faults)
            else:
                test_r.autopass()
        else:
            test_r.skip()
        in_dish.add_result(test_r)

    def test_plot_length(self, in_dish):
        '''
        Test the number of words in the plot description.
        It should be at least 20.
        '''
        test_r = TestResult('generic.story.plot.length', 5)
        sty = in_dish.item.story
        if sty is not None:
            if sty.plot is not None:
                p_str = str(sty.plot).strip()
                p_words = p_str.split()
                if len(p_words) < 20:
                    fault = Fault(FaultLevel.NOTICE,
                                  'Plot has less than 20 words')
                    test_r.result(3, StatusCode.FAIL, fault)
                else:
                    test_r.autopass()
            else:
                test_r.skip()
        else:
            test_r.skip()
        in_dish.add_result(test_r)



#!/usr/bin/env python

#
# Copyright 2023 Chris Josephes
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#

'''
Object classes related to the story/plot
of a given piece of media.
'''

# pylint: disable=too-few-public-methods

import re
from media.xml.namespaces import Namespaces
from media.data.media.contents.generic.keywords import Keywords


class Story():
    '''
    The story is the main narrative of the film or other media.
    '''
    def __init__(self, in_element):
        self.plot = None
        self.keywords = None
        self.themes = []
        self.characters = []
        self._process(in_element)

    def _process(self, in_element):
        for child in in_element:
            tagname = Namespaces.ns_strip(child.tag)
            if tagname == 'plot':
                self.plot = Plot(child)
            if tagname == 'overview':
                self.plot = Plot(child)
            if tagname == 'keywords':
                self.keywords = Keywords(child)


class Plot():
    '''
    The plot is a written summary of the narrative.
    '''
    def __init__(self, in_element):
        self.plot = ""
        self._build(in_element)

    def _build(self, in_element):
        out = []
        if in_element.text:
            out.append(in_element.text.lstrip())
        for child in in_element:
            tagname = Namespaces.ns_strip(child.tag)
            if tagname == 'chr':
                out.append(child.text.lstrip())
            if tagname == 'kw':
                out.append(child.text.lstrip())
            if child.tail:
                out.append(child.tail)
        out_string = "".join(out)
        out_string = re.sub(r'\s+', ' ', out_string)
        self.plot = out_string.rstrip()

    def __str__(self):
        return self.plot
